# 应用名称
from django.urls import re_path

app_name = 'push'
from .service_register import register
from .apis.push_apis import PushApis
from .apis.push_single_apis import PushSingleApi

register()
urlpatterns = [
    re_path(r'^single_batch_edit/?$', PushSingleApi.batch_dit, name="站内推送批量修改"),
    re_path(r'^wechat/?$', PushApis.template_send, name="模板推送"),
    re_path(r'^single_push_list/?$', PushSingleApi.list, name="模板推送列表"),
    re_path(r'^single_push_add/?$', PushSingleApi.add, name="模板推送添加"),
    re_path(r'^single_push_del/?$', PushSingleApi.delete, name="模板删除"),
    re_path(r'^single_push_edit/?$', PushSingleApi.edit, name="模板编辑"),
    re_path(r'^single_group_list/?$', PushSingleApi.group_list, name="单挑分组列表"),
]



# encoding: utf-8
"""
@project: djangoModel->__init__
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2023/8/10 8:58
"""
from orator import DatabaseManager

from config.config import JConfig

config = JConfig()
db_config = {
    config.get('main', 'driver', "mysql"): {
        'driver': config.get('main', 'driver', "mysql"),
        'host': config.get('main', 'mysql_host', "127.0.0.1"),
        'database': config.get('main', 'mysql_database', ""),
        'user': config.get('main', 'mysql_user', "root"),
        'password': config.get('main', 'mysql_password', "123456"),
        "port": config.getint('main', 'mysql_port', "3306")
    }
}


base_db = DatabaseManager(db_config)


# encoding: utf-8
"""
@project: djangoModel->service_register
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 对外开放服务调用注册白名单
@created_time: 2023/1/12 14:29
"""

import xj_push
from .push_pipeline import bx_custom_pipeline, bx_worker_pipeline, bx_visitor_pipeline,single_push_pipeline
from .utils.service_manager import ServiceManager

# 对外服务白名单
register_list = [
    {
        "name": "镖行用户通知管道",
        "service_name": "bx_custom_pipeline",
        "pointer": bx_custom_pipeline.BXCustomPipeline.process
    },
    {
        "name": "镖行镖师通知管道",
        "service_name": "bx_worker_pipeline",
        "pointer": bx_worker_pipeline.BXWorkerPipeline.process
    },
    {
        "name": "镖行游客通知管道",
        "service_name": "bx_visitor_pipeline",
        "pointer": bx_visitor_pipeline.BXVisitorPipeline.process
    },
    {
        "name": "镖行站内推送管道",
        "service_name": "bx_single_pipeline",
        "pointer": single_push_pipeline.SinglePushPipeline.process
    }
]

server_manager = ServiceManager()


# 遍历注册
def register():
    for i in register_list:
        setattr(xj_push, i["service_name"], i["pointer"])
        server_manager.put_service(route=i["service_name"], method=i["pointer"])


if __name__ == '__main__':
    register()



# _*_coding:utf-8_*_
import re


class JRecur:

    # 该方法和create_forest功能基本一致呢，区别是一个是单树，一个是多树
    @staticmethod
    def create_tree(source_list, search_root_key, search_root_value, primary_key="id", parent_key="parent_id", children_key='children'):
        """
        生成树（一颗树）
        @param source_list 需要生成的来源数据列表
        @param search_root_key 需要从列表中搜索的树根节点键名
        @param search_root_value 需要从列表中搜索的树根植
        @param primary_key 指定主键名
        @param parent_key 指定父键名
        @param children_key 指定子键名
        """
        # print("> JRecur.create_tree:", search_root_key, search_root_value)
        # 遍历列表，把数据存放在dict里
        result_tree = {}
        source_index_dict = {}  # 父ID为键名组成的源字典，用于快速索引。里面的值均为列表
        for item in source_list:
            # 查找到树根。因为不知道哪个是被搜索项，所以在遍历时顺便找一下（只是顺便，其实也可以做两次for来解决）
            if str(item.get(search_root_key, '')) == str(search_root_value):
                result_tree = item

            # 边界检查：如果没有父类别，不需要生成索引列表字典
            if not item.get(parent_key, None):
                continue
            # 父ID的字符串
            pid = str(item[parent_key])
            # 边界检查：如果索引未创建，则初始化
            if not source_index_dict.get(pid, None):
                source_index_dict[pid] = []
            # 根据父类别ID插入列表字典
            source_index_dict[pid].append(item)
        # print("> JRecur.create_tree result_tree:", result_tree)
        # print("> JRecur.create_tree parent_node:", source_index_dict)

        # 如果不能找到树根节点，则不需要做递归了
        if not result_tree:
            return {}

        # 递归树 parent_node:树的每个父节点指针, index_dict:以每个节点的父ID作为键名的字典
        def recur_node(parent_node={}, source_dict={}, pk='id', ck='children'):
            pid_str = str(parent_node[pk])
            # 孩子列表，如果源字典中没有孩子则不需要添加孩子
            if not source_dict.get(pid_str, None):
                return parent_node
            parent_node[ck] = source_dict.get(pid_str, [])
            for child in parent_node[ck]:
                recur_node(child, source_dict)
            return parent_node

        result_tree = recur_node(parent_node=result_tree, source_dict=source_index_dict, pk=primary_key, ck=children_key)
        return result_tree

    @staticmethod
    def create_forest(source_list, primary_key="id", parent_key="parent_id", children_key='children'):
        """
        生成森林（多颗树）
        @param source_list 需要生成的来源数据列表
        @param search_root_key 需要从列表中搜索的树根节点键名
        @param search_root_value 需要从列表中搜索的树根植
        @param primary_key 指定主键名
        @param parent_key 指定父键名
        @param children_key 指定子键名
        """
        # 遍历列表，把数据存放在dict里
        result_forest = []
        source_index_dict = {}  # 父ID为键名组成的源字典，用于快速索引。里面的值均为列表
        for item in source_list:
            # 查找到树根。
            if not item.get(parent_key, None) or str(item.get(parent_key, '0')) == '0':
                result_forest.append(item)
                continue  # 边界检查：如果没有父类别，不需要生成索引列表字典

            pid = str(item[parent_key])  # 父ID的字符串

            # 边界检查：如果索引未创建，则初始化
            if not source_index_dict.get(pid, None):
                source_index_dict[pid] = []

            # 根据父类别ID插入列表字典
            source_index_dict[pid].append(item)

        # 如果不能找到树根节点，则不需要做递归了
        if not result_forest or not source_index_dict:
            return []

        # 递归树 parent_node:树的每个父节点指针, index_dict:以每个节点的父ID作为键名的字典
        def recur_node(parent_node={}, source_dict={}, pk='id', ck='children'):
            pid_str = str(parent_node[pk])
            # 孩子列表，如果源字典中没有孩子则不需要添加孩子
            if not source_dict.get(pid_str, None):
                return parent_node
            parent_node[ck] = source_dict.get(pid_str, [])
            for child in parent_node[ck]:
                recur_node(child, source_dict)
            return parent_node

        for index, item in enumerate(result_forest):
            result_forest[index] = recur_node(parent_node=item, source_dict=source_index_dict, pk=primary_key, ck=children_key)

        return result_forest

    @staticmethod
    def filter_forest(source_forest=[], find_key=None, find_value=None, children_key='children', is_family_tree=False, ):
        """
        过滤森林树。支持多颗树查找，因为现实情况下被查找项可能同时属于多个家族树，所以需要用列表存放
        @param source_forest 需要过滤的森林树
        @param find_key 需要从列表中查找的树节点键名
        @param find_value 需要从列表中查找的树节点键值
        @param children_key 指定树分支键名
        @param is_family_tree 查找成功后，是否返回家族树（父亲、儿子、兄弟、祖先、后代）。True返回家族树，False返回生成子树（后代）
        """
        result_forest = []

        # 递归查找树（第一层是家族树）。判断子孙匹配则返回所在节点的生成子树
        def recur_find(node, k, v, ck):
            # 注：利用return实现匹配立即停止遍历，可巧妙减少计算遍历量 20221007 by Sieyoo
            if v and node.get(k, None) == v:
                return node
            # 如果当前节点不匹配，判断有孩子分支则继续递归，直到遍历完所有子孙节点（直至树叶）
            if node.get(ck, None):
                for it in node.get(ck):
                    cur = recur_find(it, k, v, ck)
                    if cur:
                        return cur
            # 遍历后仍不匹配，则层层回归返空值。
            return None

        # 森林是由多个家族树组成，所以最外层先做家族循环，再把匹配家族添加到结果
        for family_tree in source_forest:
            sub_tree = recur_find(family_tree, k=find_key, v=find_value, ck=children_key)
            if sub_tree:
                result_forest.append(family_tree if is_family_tree else sub_tree)

        return result_forest

    @staticmethod
    def filter_tree_values(tree_list, f_key="id", c_key="children") -> list:
        """
        遍历树中所有的节点的值
        :param tree_list: 树字典列表
        :param f_key: 检索的key
        :param c_key: 子节点的key名
        :return: list
        """
        out_list = []
        for item in tree_list:
            out_list.append(item[f_key])
            if item.get(c_key, None):
                out_list.extend(JRecur.filter_tree_values(item[c_key]))
        return out_list

    @staticmethod
    def get_filter_tree_params(params: dict = None, prefix="filter_tree_"):
        """
        解析参数中的树过滤方法
        :param params:搜索参数
        :param prefix: 参数前缀
        """
        if not isinstance(params, dict):
            return {}, None
        filter_tree_params = {}
        for k, v in params.items():
            if not re.search("^" + prefix, k):
                continue
            filter_tree_params[k.replace(prefix, "")] = v
        return filter_tree_params, None



# encoding: utf-8
"""
@project: djangoModel->push_pipeline
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 推送管道
@created_time: 2023/7/18 13:08
"""
import time

from config.config import JConfig
from .pipline_base import PipelineBase
from ..services.push_template_main_service import PushTemplateMainService
from ..utils.custom_tool import write_to_log, force_transform_type

config = JConfig()


class BXVisitorPipeline(PipelineBase):

    @staticmethod
    def process(*args, params: dict = None, **kwargs):
        """流程参数进入管道"""
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)
        params["push_current_time"] = time.strftime("%Y-%m-%d %H:%M:%S")

        user_id = params.get("user_id", None)
        if user_id:
            data, err = BXVisitorPipeline._get_user_info(user_id=user_id, prefix="")
            BXVisitorPipeline._setdefault(params=params, update_params=data)

        if not user_id:
            write_to_log(prefix="推送管道警告", content="没有用户ID无法进行推送")
            return None, "没有用户ID无法进行推送"

        # 开始推送
        push_type = params.get("push_type")
        push_template_value = params.get("push_template_value")
        if push_type and push_template_value and params:
            try:
                data, err = PushTemplateMainService.template_send(
                    type=push_type,
                    value=push_template_value,
                    replacements=params,
                    user_id=user_id
                )
                # TODO 推送人数加一
            except Exception as e:
                # TODO 推送失败人数加一
                write_to_log(prefix="推送管道调用推送服务失败：PushTemplateMainService", err_obj=e)
        return params, None



from pathlib import Path

from django.forms.models import model_to_dict

from main.settings import BASE_DIR
from utils.custom_tool import write_to_log, format_params_handle
from xj_user.services.login_service import LoginService
from xj_user.utils.wechat_sign import applet_subscribe_message, subscribe_message
from ..models import PushTemplate
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.utility_method import replace_values

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

payment_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
payment_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = payment_main_config_dict.wechat_merchant_app_id or payment_module_config_dict.wechat_merchant_app_id or ""
sub_app_secret = payment_main_config_dict.wechat_merchant_app_secret or payment_module_config_dict.wechat_merchant_app_secret or ""
wechat_merchant_name = payment_main_config_dict.wechat_merchant_name or payment_module_config_dict.wechat_merchant_name or ""

subscription_app_id = payment_main_config_dict.wechat_subscription_app_id or payment_module_config_dict.wechat_subscription_app_id or ""
subscription_app_secret = payment_main_config_dict.wechat_subscription_app_secret or payment_module_config_dict.wechat_subscription_app_secret or ""

app_app_id = payment_main_config_dict.wechat_app_app_id or payment_module_config_dict.wechat_app_app_id or ""
app_app_secret = payment_main_config_dict.wechat_app_app_secret or payment_module_config_dict.wechat_app_app_secret or ""


class PushTemplateMainService:

    # 微信模板发送
    @staticmethod
    def template_send(type, value, user_id, replacements=None):
        """
        微信模板发送
        :param touser: 要推送的人 openid
        :param replacements: 要替换的模板参数
        :param type: 模板类型
        :return: param_dict
        """
        new_data = {}
        template_set = PushTemplate.objects.filter(value=value).first()
        if not template_set:
            write_to_log(
                prefix="模板记录",
                content="模板关键词:" + str(value) + "不存在",
            )
            return None, "模板记录不存在"

        template_set = model_to_dict(template_set)
        data = format_params_handle(
            param_dict=format_params_handle(
                param_dict=replacements,
                alias_dict=template_set.get("alias_params"),
            ),
            filter_filed_list=list(template_set.get("alias_params").values()),
        )
        write_to_log(prefix="推送记录收录：", content="模板变量:" + str(data or {}) + " 推送用户：" + str(user_id) + "推送模板:" + str(value))

        template = template_set.get("template")
        if type in ['subscribe', 'applet']:
            new_data = replace_values(data, template)
        if type == "sms":
            wechat_user_info = subscribe_message(new_data)
        elif type == "subscribe":
            sso, err = LoginService.sso_record(user_id, subscription_app_id)
            if err:
                write_to_log(
                    prefix="公众号获取单点登录信息",
                    content="单点登录信息:" + str(user_id) + "不存在",
                    err_obj=err
                )
            new_data['touser'] = sso.get("sso_unicode", "")
            # wechat_user_info = subscribe_message(new_data)
        elif type == "applet":
            sso, err = LoginService.sso_record(user_id, sub_appid)
            if err:
                write_to_log(
                    prefix="小程序获取单点登录信息",
                    content="单点登录信息:" + str(user_id) + ";" + str(sub_appid) + "不存在",
                    err_obj=err
                )
                return None, err
            new_data['touser'] = sso.get("sso_unicode", "")
            wechat_user_info = applet_subscribe_message(new_data)
            write_to_log(prefix="微信推送结果日志", content=wechat_user_info)

        # TODO 推送成功人数加一
        # print(new_data)
        return None, None



from django.db import models


class PushPlan(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    role_id = models.IntegerField(verbose_name='权限id', help_text='')
    group_id = models.IntegerField(verbose_name='分组id', help_text='')
    user_id_list = models.JSONField(verbose_name='用户id列表', blank=True, null=True, help_text='')
    template_id = models.IntegerField(verbose_name='权限ID', help_text='')
    node_to_action_id = models.IntegerField(verbose_name='节点对动作ID', help_text='')
    plan_push_circle = models.CharField(verbose_name='计划周期，参考contab', max_length=50, help_text='')
    plan_push_time = models.DateTimeField(verbose_name='推送时间', help_text='')
    created_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True)

    class Meta:
        db_table = 'push_plan'
        verbose_name_plural = "推送-推送流程"

    def __str__(self):
        return f"{self.id}"


class PushPush(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    templet_id = models.IntegerField(verbose_name='模板id', primary_key=False, help_text='')
    plan_id = models.IntegerField(verbose_name='计划id', primary_key=False, help_text='')
    title = models.CharField(verbose_name='标题', max_length=128)
    content = models.CharField(verbose_name='内容', max_length=128)
    files = models.JSONField(verbose_name='文件', blank=True, null=True, help_text='')
    push_total = models.IntegerField(verbose_name='本次推送用户数量', primary_key=False, help_text='')
    send_success = models.IntegerField(verbose_name='发送成功的用户', primary_key=False, help_text='')
    send_time = models.DateTimeField(verbose_name='开始发送时间')
    created_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True)

    class Meta:
        db_table = 'push_push'
        verbose_name_plural = "推送-推送主表"

    def __str__(self):
        return f"{self.id}"


class PushRecord(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    push_id = models.IntegerField(verbose_name='推送ID', primary_key=False, help_text='')
    to_user_id = models.IntegerField(verbose_name='发送的用户ID', primary_key=False, help_text='')
    is_receive = models.CharField(verbose_name='是否送达', max_length=128)
    created_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True)

    class Meta:
        db_table = 'push_record'
        verbose_name_plural = "推送-推送记录"

    def __str__(self):
        return f"{self.id}"


class PushTemplate(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    value = models.CharField(verbose_name='模板搜索关键词', max_length=128, blank=True, null=True, help_text='')
    template = models.CharField(verbose_name='内容推送模板，变量使用{}包裹', max_length=128, blank=True, null=True, help_text='')
    default_params = models.JSONField(verbose_name='模板变量', blank=True, null=True, help_text='')
    alias_params = models.JSONField(verbose_name='模板别名', blank=True, null=True, help_text='')
    title = models.CharField(verbose_name='发送标题', max_length=128, blank=True, null=True, help_text='')
    send_type = models.CharField(verbose_name='模板类型', max_length=128, blank=True, null=True, help_text='')
    files = models.JSONField(verbose_name='模板变量', blank=True, null=True, help_text='')
    push_times = models.IntegerField(verbose_name='推送次数', primary_key=False, help_text='')
    push_success = models.IntegerField(verbose_name='推送成功人数', primary_key=False, help_text='')
    push_total = models.IntegerField(verbose_name='推送模板人数', primary_key=False, help_text='')
    created_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True)
    enabled = models.IntegerField(verbose_name='是否启用', default=0)

    class Meta:
        db_table = 'push_template'
        verbose_name_plural = "推送-推送模板"

    def __str__(self):
        return f"{self.id}"


class PushSingle(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    to_user_id = models.IntegerField(verbose_name='发送用户ID', help_text='')
    source_code = models.CharField(verbose_name='发送用户ID', max_length=50, help_text='')
    title = models.CharField(verbose_name='发送用户ID', max_length=255, help_text='')
    content = models.TextField(verbose_name='发送用户ID', help_text='')
    template = models.ForeignKey("PushTemplate", verbose_name="模板变量", help_text="", null=True, blank=True, on_delete=models.DO_NOTHING)
    thread_id = models.IntegerField("PushTemplate", help_text="", null=True, blank=True)
    template_params = models.JSONField(verbose_name="模板变量", help_text="")
    link = models.CharField(verbose_name='发送类型', max_length=500, null=True, blank=True, help_text='')
    is_jump_link = models.BooleanField(verbose_name='发送类型', default=0, null=True, blank=True, help_text='')
    send_type = models.TextField(verbose_name='发送类型', help_text='')
    files = models.JSONField(verbose_name='文件集合', help_text='')
    is_read = models.IntegerField(verbose_name='是否阅读', default=0, help_text='')
    is_delete = models.IntegerField(verbose_name='是否删除', default=0, help_text='')
    snapshot = models.JSONField(verbose_name='快照', null=True, blank=True, default={})
    created_time = models.DateTimeField(verbose_name='创建时间', help_text='')

    class Meta:
        db_table = 'push_single'
        verbose_name_plural = "推送-站内推送"

    def __str__(self):
        return f"{self.id}"



from django.contrib import admin
# 引入用户平台
from .models import *


# #
#
class PushPushAdmin(admin.ModelAdmin):
    fields = (
        'id', 'templet_id', 'plan_id', 'title', 'content', 'files', 'push_total', 'send_success', 'send_time',
        'created_time')
    list_display = (
        'id', 'templet_id', 'plan_id', 'title', 'content', 'files', 'push_total', 'send_success', 'send_time',
        'created_time')
    search_fields = (
        'templet_id', 'plan_id', 'invoice_type', 'title')
    readonly_fields = ['id']


# #
#
class PushPlanAdmin(admin.ModelAdmin):
    fields = (
        'id', 'role_id', 'group_id', 'user_id_list', 'node_to_action_id', 'created_time')
    list_display = (
        'id', 'role_id', 'group_id', 'user_id_list', 'node_to_action_id', 'created_time')
    search_fields = (
        'role_id', 'plan_id', 'group_id')
    readonly_fields = ['id']


class PushRecordAdmin(admin.ModelAdmin):
    fields = (
        'id', 'push_id', 'to_user_id', 'is_receive', 'created_time')
    list_display = (
        'id', 'push_id', 'to_user_id', 'is_receive', 'created_time')
    search_fields = (
        'push_id',)
    readonly_fields = ['id']


class PushTemplateAdmin(admin.ModelAdmin):
    fields = (
        'id', 'value', 'template', 'default_params', 'alias_params', 'title', 'send_type', 'files', 'push_times',
        'push_success',
        'push_total', 'created_time')
    list_display = (
        'id', 'value', 'template', 'default_params', 'alias_params', 'title', 'send_type', 'files', 'push_times',
        'push_success',
        'push_total', 'created_time')
    search_fields = (
        'title',)
    readonly_fields = ['id']


#
# #
admin.site.register(PushPush, PushPushAdmin)
admin.site.register(PushPlan, PushPlanAdmin)
admin.site.register(PushRecord, PushRecordAdmin)
admin.site.register(PushTemplate, PushTemplateAdmin)



# encoding: utf-8
"""
@project: djangoModel->push_pipeline
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 推送管道
@created_time: 2023/7/18 13:08
"""
import time

from config.config import JConfig
from .bx_custom_pipeline import BXCustomPipeline
from .pipline_base import PipelineBase
from ..services.push_template_main_service import PushTemplateMainService
from ..utils.custom_tool import force_transform_type, write_to_log, dynamic_load_class

config = JConfig()


class BXWorkerPipeline(PipelineBase):

    @staticmethod
    def get_record(enroll_id, prefix="record"):
        record_info = {}
        enroll_id, err = force_transform_type(variable=enroll_id, var_type="int")
        EnrollRecordServices, import_err = dynamic_load_class(import_path="xj_enroll.service.enroll_record_serivce", class_name="EnrollRecordServices")
        if enroll_id and not import_err:
            data, err = EnrollRecordServices.record_detail(
                search_params={"enroll_id": enroll_id, "exclude_code": 124},
            )
            if not err and isinstance(data, dict):
                record_info = data

        return {prefix + k: v for k, v in record_info.items()}, None

    @staticmethod
    def process(*args, params: dict = None, **kwargs):
        """流程参数进入管道"""
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)
        params["push_current_time"] = time.strftime("%Y-%m-%d %H:%M:%S")

        # 获取报名相关的信息
        enroll_id = params.get("enroll_id", None)
        if enroll_id:
            # 报名主表和项目信息
            data, err = BXWorkerPipeline._get_enroll_info(enroll_id=enroll_id, prefix="enroll_")
            BXCustomPipeline._setdefault(params=params, update_params=data)
            # 报名记录信息
            data, err = BXWorkerPipeline.get_record(enroll_id=enroll_id, prefix="record_")
            BXCustomPipeline._setdefault(params=params, update_params=data)

        # 获取开票相关的信息
        invoice_id = params.get("invoice_id", None)
        if invoice_id:
            data, err = BXWorkerPipeline.__get_invoice_info(invoice_id=invoice_id)
            BXWorkerPipeline._setdefault(params=params, update_params=data)

        # 获取财务相关的信息
        finance_id = params.get("finance_id", None)
        order_no = params.get("order_no", None)
        transact_no = params.get("transact_no", None)
        if finance_id or transact_no or order_no:
            data, err = BXCustomPipeline._get_finance_info(finance_id=finance_id, order_no=order_no, transact_no=transact_no)
            BXCustomPipeline._setdefault(params=params, update_params=data)

        # 开始推送
        push_type = params.get("push_type")
        push_template_value = params.get("push_template_value")
        if not params.get("record_user_id"):
            write_to_log(prefix="推送管道警告", content="该项目还没有镖师承接订单，enroll_id为" + str(enroll_id or "None"))
            return None, "该项目还没有镖师承接订单"
        if push_type and push_template_value and params:
            try:
                write_to_log(
                    prefix="发起推送",
                    content="push_type:" + str(push_type) + " push_template_value:" + str(push_template_value) + " user_id:" + str(params.get("record_user_id", ""))
                )
                data, err = PushTemplateMainService.template_send(
                    type=push_type,
                    value=push_template_value,
                    replacements=params,
                    user_id=params.get("record_user_id", 0)
                )
                write_to_log(prefix="推送结果", content="data:" + str(data) + " ; err" + str(err or ""))
                return None, None
                # TODO 推送人数加一
            except Exception as e:
                # TODO 推送失败人数加一
                write_to_log(prefix="推送管道调用推送服务失败：PushTemplateMainService", err_obj=e)
        return params, None



# encoding: utf-8
"""
@project: djangoModel->push_template
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 推送模板模型
@created_time: 2023/8/10 9:05
"""
from .base_model import BaseModel


class PushTemplate(BaseModel):
    __table__ = 'push_template'
    __guarded__ = ['id', 'created_time']




# encoding: utf-8
"""
@project: djangoModel->push_pipeline
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 推送管道
@created_time: 2023/7/18 13:08
"""
import time

from orator import DatabaseManager

from config.config import JConfig
from .pipline_base import PipelineBase
from ..services.push_template_main_service import PushTemplateMainService
from ..utils.custom_tool import force_transform_type, write_to_log

config = JConfig()
db_config = {
    config.get('main', 'driver', "mysql"): {
        'driver': config.get('main', 'driver', "mysql"),
        'host': config.get('main', 'mysql_host', "127.0.0.1"),
        'database': config.get('main', 'mysql_database', ""),
        'user': config.get('main', 'mysql_user', "root"),
        'password': config.get('main', 'mysql_password', "123456"),
        "port": config.getint('main', 'mysql_port', "3306")
    }
}
db = DatabaseManager(db_config)


class BXCustomPipeline(PipelineBase):

    @staticmethod
    def process(*args, params: dict = None, **kwargs):
        """流程参数进入管道"""
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)
        params["push_current_time"] = time.strftime("%Y-%m-%d %H:%M:%S")
        # # 获取报名相关的信息
        enroll_id = params.get("enroll_id", None)
        if enroll_id:
            data, err = BXCustomPipeline._get_enroll_info(enroll_id=enroll_id, prefix="enroll_")
            BXCustomPipeline._setdefault(params=params, update_params=data)

        category_id = params.get("category_id", None)
        if category_id:
            category_obj = db.table("thread_category").select_raw("""
                value as category_value,
                name as category_name,
                need_auth,
                platform_code,
                parent_id  as category_parent_id
            """).where("id", '=', category_id).first()
            BXCustomPipeline._setdefault(params=params, update_params=category_obj)

        classify_id = params.get("classify_id", None)
        if classify_id:
            classify_obj = db.table("thread_classify").select_raw("""
                value as classify_value,
                name as classify_name,
                category_id as classify_category_id,
                parent_id  as classify_parent_id
            """).where("id", '=', classify_id).first()
            BXCustomPipeline._setdefault(params=params, update_params=classify_obj)

        enroll_user_id = params.get("enroll_user_id", None)
        if enroll_user_id:
            data, err = BXCustomPipeline._get_user_info(user_id=enroll_user_id, prefix="enroll_user_")
            BXCustomPipeline._setdefault(params=params, update_params=data)

        # # 获取开票相关的信息
        invoice_id = params.get("invoice_id", None)
        if invoice_id:
            data, err = BXCustomPipeline.__get_invoice_info(invoice_id=invoice_id)
            BXCustomPipeline._setdefault(params=params, update_params=data)

        # 获取财务相关的信息
        finance_id = params.get("finance_id", None)
        order_no = params.get("order_no", None)
        transact_no = params.get("transact_no", None)
        if finance_id or transact_no or order_no:
            data, err = BXCustomPipeline._get_finance_info(finance_id=finance_id, order_no=order_no, transact_no=transact_no)
            BXCustomPipeline._setdefault(params=params, update_params=data)

        push_type = params.get("push_type")
        push_template_value = params.get("push_template_value")
        # print("BXCustomPipeline.process params:", params)
        if push_type and push_template_value and params:
            try:
                data, err = PushTemplateMainService.template_send(
                    type=push_type,
                    value=push_template_value,
                    replacements=params,
                    user_id=params.get("enroll_user_id")
                )
            except Exception as e:
                write_to_log(prefix="推送管道调用推送服务失败：PushTemplateMainService", err_obj=e)

        return None, None



# encoding: utf-8
"""
@project: djangoModel->single_push_pipline
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 单挑推送管道
@created_time: 2023/8/23 13:13
"""
from django.db.models import F

from xj_enroll.models import EnrollSubitemRecord
from xj_push.push_pipeline.pipline_base import PipelineBase
from xj_push.services.push_single_service import PushSingleService
from ..utils.custom_tool import force_transform_type, dynamic_load_class, write_to_log


class SinglePushPipeline(PipelineBase):
    worker_notice_title = {
        "enroll": {
            "title": "报名成功",
            "content": "该项目您已报名",
        },
        "appoint": {
            "title": "抢单成功",
            "content": "恭喜您成功抢到订单，快去制作吧",
        },
        "appoint_fail": {
            "title": "报名失败",
            "content": "客户未派单给您，继续努力~",
        },
        "upload": {
            "title": "资料审核中",
            "content": "您已提交应标资料，平台审核中",
        },
        "check_success": {
            "title": "待客户验收",
            "content": "您上传的资料平台已审核通过",
        },
        "check_fail": {
            "title": "派台审核不通过",
            "content": "请您修改标书文件后重新上传",
        },
        "accept_success": {
            "title": "完成订单",
            "content": "恭喜您的，您的订单完成验收",
        },
        "accept_fail": {
            "title": "客户拒绝订单",
            "content": "请修改后再次上传标书文件",
        },
        "cancel_publish": {
            "title": "订单关闭",
            "content": "客户已取消订单",
        },
        "cancel_enroll": {
            "title": "取消报名成功",
            "content": "您已取消报名",
        },
    }

    custom_notice_title = {
        "publish": {
            "title": "订单报名中",
            "content": "您发布的新订单正在报名中",
        },
        "enroll": {
            "title": "订单派单",
            "content": "您的订单有新的镖师报名",
        },
        "appoint": {
            "title": "已派单，等待付款",
            "content": "付款后镖师收到通知，开始制作",
        },
        "pay": {
            "title": "等待镖师上传标书",
            "content": "订单完成付款，待镖师提交标书",
        },
        "upload": {
            "title": "平台审核资料",
            "content": "镖师已上传项目标书，平台审核中",
        },
        "check_success": {
            "title": "标书待验收",
            "content": "平台完成标书审核",
        },
        "check_fail": {
            "title": "等待镖师上传标书",
            "content": "审核失败，待镖师重新提交标书",
        },
        "accept_success": {
            "title": "订单完成",
            "content": "项目标书完成验收",
        },
        "accept_fail": {
            "title": "拒绝验收",
            "content": "验收失败，待镖师重新提交标书",
        },
        "cancel_publish": {
            "title": "订单关闭",
            "content": "您已取消该订单",
        },
    }

    @staticmethod
    def process(*args, params: dict = None, **kwargs):
        # 获取报名相关的信息
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)
        enroll_id = params.get("enroll_id", None)
        if not enroll_id:
            return None, "报名ID错误"

        # 导入依赖
        Enroll, enroll_import_err = dynamic_load_class(import_path="xj_enroll.models", class_name="Enroll")
        EnrollRecord, record_import_err = dynamic_load_class(import_path="xj_enroll.models", class_name="EnrollRecord")
        if enroll_import_err or record_import_err:
            write_to_log(prefix="站内推送管道异常", content="请安装报名模块")
            return None, None

        # write_to_log(prefix="站内推送管道入参", content=params)
        # 获取相关信息
        enroll_info = Enroll.objects.filter(id=enroll_id).values().first()
        thread_id = enroll_info.get("thread_id")
        enroll_user_id = enroll_info.get("user_id")
        add_params = {"thread_id": thread_id, "source_code": "订单动态", "snapshot": params}

        # print("params", params)
        # ------------------------- section 管道挂油，管道分流 start ------------------------
        # 发布的订单后通知客户
        if params.get("push_action") == "publish":
            add_params["title"] = SinglePushPipeline.custom_notice_title["publish"]["title"]
            add_params["content"] = SinglePushPipeline.custom_notice_title["publish"]["content"]
            add_params["to_user_id"] = enroll_user_id
            data, err = PushSingleService.add(params=add_params)

        # 镖师报名相互通知
        elif params.get("push_action") == "enroll":
            # 提醒客户有人报名
            add_params["content"] = SinglePushPipeline.custom_notice_title["enroll"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["enroll"]["title"]
            add_params["to_user_id"] = enroll_user_id
            data, err = PushSingleService.add(params=add_params)

            # 体校镖师报名成功
            record_info = EnrollRecord.objects.filter(id=params.get("record_id")).values().first()
            if record_info:
                add_params["content"] = SinglePushPipeline.worker_notice_title["enroll"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["enroll"]["title"]
                add_params["to_user_id"] = record_info.get("user_id")
                data, err = PushSingleService.add(params=add_params)

        # 镖师取消报名
        elif params.get("push_action") == "cancel":
            # # 提醒客户有人报名
            # add_params["content"] = SinglePushPipeline.custom_notice_title["cancel"]["content"]
            # add_params["title"] = SinglePushPipeline.custom_notice_title["cancel"]["title"]
            # add_params["to_user_id"] = enroll_user_id
            # data, err = PushSingleService.add(params=add_params)

            # 提醒镖师客户取消了订单
            record_info = EnrollRecord.objects.filter(id=params.get("record_id")).values().first()
            if record_info:
                add_params["content"] = SinglePushPipeline.worker_notice_title["cancel_enroll"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["cancel_enroll"]["title"]
                add_params["to_user_id"] = record_info.get("user_id")
                data, err = PushSingleService.add(params=add_params)

        # 用户取消订单通知所有的报名用户
        elif params.get("push_action") == "cancel_publish":
            # 通知所有报名的镖师
            add_params["content"] = SinglePushPipeline.worker_notice_title["cancel_publish"]["content"]
            add_params["title"] = SinglePushPipeline.worker_notice_title["cancel_publish"]["title"]
            record_infos = list(EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values())
            for i in record_infos:
                if not i.get("user_id"):
                    continue
                add_params["snapshot"]["record_id"] = i.get("id")
                add_params["to_user_id"] = i.get("user_id")
                PushSingleService.add(params=add_params)

            # 通知客户取消成功
            add_params["content"] = SinglePushPipeline.custom_notice_title["cancel_publish"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["cancel_publish"]["title"]
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

        # 指派通知
        elif params.get("push_action") == "appoint":
            # 通知客户指派成功
            add_params["content"] = SinglePushPipeline.custom_notice_title["appoint"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["appoint"]["title"]
            add_params["to_user_id"] = enroll_user_id
            data, err = PushSingleService.add(params=add_params)

            # 通知镖师您已被指派
            record_info = EnrollRecord.objects.filter(id=params.get("record_id")).values().first()
            if record_info:
                add_params["content"] = SinglePushPipeline.worker_notice_title["appoint"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["appoint"]["title"]
                add_params["to_user_id"] = record_info.get("user_id")
                data, err = PushSingleService.add(params=add_params)

            # 没有指派的镖师给予提示
            un_appoint_records = list(EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(id=params.get("record_id")).values())
            for i in un_appoint_records:
                add_params["content"] = SinglePushPipeline.worker_notice_title["appoint_fail"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["appoint_fail"]["title"]
                add_params["to_user_id"] = i.get("user_id")
                data, err = PushSingleService.add(params=add_params)

        # 已支付通知客户
        elif params.get("push_action") == "payed":
            add_params["content"] = SinglePushPipeline.custom_notice_title["payed"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["payed"]["title"]
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

        # 镖师上传
        elif params.get("push_action") == "upload":
            # 通知客户
            add_params["content"] = SinglePushPipeline.custom_notice_title["upload"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["upload"]["title"]
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            subitem_record = EnrollSubitemRecord.objects.annotate(
                record_user_id=F("enroll_record__user_id")
            ).filter(id=params.get("id")).values("record_user_id", "enroll_record_id", "id").first()
            if subitem_record:
                add_params["content"] = SinglePushPipeline.worker_notice_title["upload"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["upload"]["title"]
                add_params["to_user_id"] = subitem_record.get("record_user_id")
                add_params["snapshot"]["record_id"] = subitem_record.get("enroll_record_id")
                PushSingleService.add(params=add_params)

        # 标书初审通过
        elif params.get("push_action") == "check_success":
            # 通知客户
            add_params["content"] = SinglePushPipeline.custom_notice_title["check_success"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["check_success"]["title"]
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            bx_worker = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values().first()
            if bx_worker:
                add_params["content"] = SinglePushPipeline.worker_notice_title["check_success"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["check_success"]["title"]
                add_params["to_user_id"] = bx_worker.get("user_id")
                add_params["snapshot"]["record_id"] = bx_worker.get("id")
                add_params["snapshot"]["worker_status_code"] = 234
                PushSingleService.add(params=add_params)

        # 标书初审失败
        elif params.get("push_action") == "check_fail":
            # 通知客户
            add_params["content"] = SinglePushPipeline.custom_notice_title["check_fail"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["check_fail"]["title"]
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            bx_worker = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values().first()
            if bx_worker:
                add_params["content"] = SinglePushPipeline.worker_notice_title["check_fail"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["check_fail"]["title"]
                add_params["to_user_id"] = bx_worker.get("user_id")
                add_params["snapshot"]["record_id"] = bx_worker.get("id")
                PushSingleService.add(params=add_params)

        # 标书初审通过
        elif params.get("push_action") == "accept_success":
            add_params["content"] = SinglePushPipeline.custom_notice_title["accept_success"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["accept_success"]["title"]
            # 通知客户
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            bx_worker = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values().first()
            if bx_worker:
                add_params["content"] = SinglePushPipeline.worker_notice_title["accept_success"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["accept_success"]["title"]
                add_params["snapshot"]["record_id"] = bx_worker.get("id")
                add_params["to_user_id"] = bx_worker.get("user_id")
                PushSingleService.add(params=add_params)

        # 标书验收失败
        elif params.get("push_action") == "accept_fail":
            add_params["content"] = SinglePushPipeline.custom_notice_title["accept_fail"]["content"]
            add_params["title"] = SinglePushPipeline.custom_notice_title["accept_fail"]["title"]
            # 通知客户
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            bx_worker = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values().first()
            if bx_worker:
                add_params["content"] = SinglePushPipeline.worker_notice_title["accept_fail"]["content"]
                add_params["title"] = SinglePushPipeline.worker_notice_title["accept_fail"]["title"]
                add_params["to_user_id"] = bx_worker.get("user_id")
                add_params["snapshot"]["record_id"] = bx_worker.get("id")
                PushSingleService.add(params=add_params)

        # ------------------------- section 管道挂油 end  ------------------------
        return None, None
